/*written by: Alex Sood
debugged by: Alex Sood*/
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import java.lang.InterruptedException;
import java.sql.SQLException;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.JTextField;
public class AccountScreen extends JDialog implements ActionListener
{
	public class AccountScreenUpdater extends Thread
	{
		public AccountScreenUpdater()
		{
			super("AccountScreenUpdater");
			setDaemon(true);
		}
		
		public void run()
		{
			while(true)
			{
				try
				{
					fillOrders();
					dispArea.setText(port.toString());
					sleep(1000);
				}
				catch(InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		private void fillOrders()
		{
			Order ord = pending;
			while(ord != null)
			{
				try
				{
					if(ord.getType().equals("market") && ord.getBuyOrSell() == 0) 
					{
						port.buy(ord.getTicker(), ord.getNumShares());
						if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
						else pending = ord.getNext();
						if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
					}
					if(ord.getType().equals("market") && ord.getBuyOrSell() == 1)
					{
						port.sell(ord.getTicker(), ord.getNumShares());
						if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
						else pending = ord.getNext();
						if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
					}
					if(ord.getType().equals("limit") && ord.getBuyOrSell() == 0 && StockList.getPrice(ord.getTicker()) <= ord.getPrice())
					{
						port.buy(ord.getTicker(), ord.getNumShares());
						if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
						else pending = ord.getNext();
						if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
					}
					if(ord.getType().equals("limit") && ord.getBuyOrSell() == 1 && StockList.getPrice(ord.getTicker()) >= ord.getPrice())
					{
						port.sell(ord.getTicker(), ord.getNumShares());
						if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
						else pending = ord.getNext();
						if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
					}
					if(ord.getType().equals("stoploss") && StockList.getPrice(ord.getTicker()) <= ord.getPrice())
					{
						port.sell(ord.getTicker(), ord.getNumShares());
						if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
						else pending = ord.getNext();
						if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
					}
					if(ord.getType().equals("trailstop") && (1 - StockList.getPrice(ord.getTicker())/ord.getPrice()) >= ord.getPercent())
					{
						port.sell(ord.getTicker(), ord.getNumShares());
						if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
						else pending = ord.getNext();
						if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
					}
				}
				catch(SQLException e)
				{
					if(ord.getPrev() != null) ord.getPrev().setNext(ord.getNext());
					else pending = ord.getNext();
					if(ord.getNext() != null) ord.getNext().setPrev(ord.getPrev());
				}
				ord = ord.getNext();
			}
		}
	}
	
	public class BuySellScreen extends JDialog implements ActionListener
	{
		private JPanel inPanel = new JPanel();
		private JPanel radPanel = new JPanel();
		private JPanel buttonPanel = new JPanel();
		private JTextField stockField = new JTextField(15);
		private JTextField numField = new JTextField(15);
		private JTextField priceField = new JTextField(15);
		private JTextField percentField = new JTextField(15);
		private JRadioButton market = new JRadioButton("Market Order");
		private JRadioButton limit = new JRadioButton("Limit Order");
		private JRadioButton stopLoss = new JRadioButton("Stop Loss Order");
		private JRadioButton trailStop = new JRadioButton("Trailing Stop Order");
		private JButton buy = new JButton("Buy");
		private JButton sell = new JButton("Sell");
		private JButton cancel = new JButton("Cancel");
		private JLabel stockLabel = new JLabel("Enter stock ticker...");
		private JLabel numLabel = new JLabel("...and number of shares");
		private JLabel priceLabel = new JLabel("Enter price limit.");
		private JLabel percentLabel = new JLabel("Enter percent limit.");
		
		public BuySellScreen()
		{
			super((JFrame)(null), true);
			setBounds(300,300,500,200);
			setDefaultCloseOperation(DISPOSE_ON_CLOSE);
			getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
			getContentPane().add(inPanel);
			getContentPane().add(radPanel);
			getContentPane().add(buttonPanel);
			inPanel.setLayout(new GridLayout(4,2,5,5));
			inPanel.add(stockField);
			inPanel.add(priceField);
			inPanel.add(stockLabel);
			inPanel.add(priceLabel);
			inPanel.add(numField);
			inPanel.add(percentField);
			inPanel.add(numLabel);
			inPanel.add(percentLabel);
			radPanel.setLayout(new BoxLayout(radPanel, BoxLayout.X_AXIS));
			radPanel.add(market);
			radPanel.add(limit);
			radPanel.add(stopLoss);
			radPanel.add(trailStop);
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			buttonPanel.add(buy);
			buttonPanel.add(sell);
			buttonPanel.add(cancel);
			priceField.setFocusable(false);
			priceField.setVisible(false);
			priceLabel.setVisible(false);
			percentField.setFocusable(false);
			percentField.setVisible(false);
			percentLabel.setVisible(false);
			ButtonGroup bg = new ButtonGroup();
			bg.add(market);
			bg.add(limit);
			bg.add(stopLoss);
			bg.add(trailStop);
			market.setSelected(true);
			market.addActionListener(this);
			limit.setSelected(false);
			limit.addActionListener(this);
			stopLoss.setSelected(false);
			stopLoss.addActionListener(this);
			trailStop.setSelected(false);
			trailStop.addActionListener(this);
			buy.addActionListener(this);
			sell.addActionListener(this);
			cancel.addActionListener(this);
			setVisible(true);
		}
		
		public void actionPerformed(ActionEvent ev)
		{
			if(ev.getSource() == market)
			{
				priceField.setText("");
				percentField.setText("");
				priceField.setFocusable(false);
				priceField.setVisible(false);
				priceLabel.setVisible(false);
				percentField.setFocusable(false);
				percentField.setVisible(false);
				percentLabel.setVisible(false);
			}
			if(ev.getSource() == limit)
			{
				if(!(limit.isSelected())) return;
				priceField.setFocusable(true);
				priceField.setVisible(true);
				priceLabel.setVisible(true);
				percentField.setText("");
				percentField.setFocusable(false);
				percentField.setVisible(false);
				percentLabel.setVisible(false);
			}
			if(ev.getSource() == stopLoss)
			{
				if(!(stopLoss.isSelected())) return;
				priceField.setFocusable(true);
				priceField.setVisible(true);
				priceLabel.setVisible(true);
				percentField.setText("");
				percentField.setFocusable(false);
				percentField.setVisible(false);
				percentLabel.setVisible(false);
			}
			if(ev.getSource() == trailStop)
			{
				if(!(trailStop.isSelected())) return;
				percentField.setFocusable(true);
				percentField.setVisible(true);
				percentLabel.setVisible(true);
				priceField.setText("");
				priceField.setFocusable(false);
				priceField.setVisible(false);
				priceLabel.setVisible(false);
			}
			if(ev.getSource() == buy)
			{
				if(stopLoss.isSelected())
				{
					new ErrBox("Must select Sell for Stop Loss order.");
					return;
				}
				if(trailStop.isSelected())
				{
					new ErrBox("Must select Sell for Trailing Stop order.");
					return;
				}
				String tk = stockField.getText().toUpperCase();
				if(tk.length() == 0)
				{
					new ErrBox("Please enter stock ticker.");
					return;
				}
				double a = -1;
				try
				{
					a = Double.parseDouble(numField.getText());
				}
				catch(NumberFormatException e)
				{
					new ErrBox("Please enter number of shares.");
					return;
				}
				if(market.isSelected())
				{
					Order ord = new Order("market", tk, 0, a, -1, -1);
					ord.setNext(pending);
					if(pending != null) pending.setPrev(ord);
					pending = ord;
				}
				if(limit.isSelected())
				{
					double prc = -1;
					try
					{
						prc = Double.parseDouble(priceField.getText());
					}
					catch(NumberFormatException e)
					{
						new ErrBox("Please enter price limit");
						return;
					}
					Order ord = new Order("limit", tk, 0, a, prc, -1);
					ord.setNext(pending);
					if(pending != null) pending.setPrev(ord);
					pending = ord;
				}
			}
			if(ev.getSource() == sell)
			{
				String tk = stockField.getText().toUpperCase();
				if(tk.length() == 0)
				{
					new ErrBox("Please enter stock ticker.");
					return;
				}
				double a  = -1;
				try
				{
					a = Double.parseDouble(numField.getText());
				}
				catch(NumberFormatException e)
				{
					new ErrBox("Please enter number of shares.");
					return;
				}
				if(market.isSelected())
				{
					Order ord = new Order("market", tk, 1, a, -1, -1);
					ord.setNext(pending);
					if(pending != null) pending.setPrev(ord);
					pending = ord;
				}
				if(limit.isSelected())
				{
					double prc = -1;
					try
					{
						prc = Double.parseDouble(priceField.getText());
					}
					catch(NumberFormatException e)
					{
						new ErrBox("Please enter price limit");
						return;
					}
					Order ord = new Order("limit", tk, 1, a, prc, -1);
					ord.setNext(pending);
					if(pending != null) pending.setPrev(ord);
					pending = ord;
				}
				if(stopLoss.isSelected())
				{
					double prc = -1;
					try
					{
						prc = Double.parseDouble(priceField.getText());
					}
					catch(NumberFormatException e)
					{
						new ErrBox("Please enter price limit");
						return;
					}
					Order ord = new Order("stoploss", tk, 1, a, prc, -1);
					ord.setNext(pending);
					if(pending != null) pending.setPrev(ord);
					pending = ord;
				}
				if(trailStop.isSelected())
				{
					double prc = -1;
					try
					{
						prc = StockList.getPrice(tk);
					}
					catch(SQLException e) { return; }
					double pct = -1;
					try
					{
						pct = Double.parseDouble(percentField.getText());
					}
					catch(NumberFormatException e)
					{
						new ErrBox("Please enter percent limit");
						return;
					}
					Order ord = new Order("trailstop", tk, 1, a, prc, pct);
					ord.setNext(pending);
					if(pending != null) pending.setPrev(ord);
					pending = ord;
				}
			}
			if(ev.getSource() == cancel) dispose();
		}
	}
	
	private JPanel dispPanel = new JPanel();
	private JPanel butPanel = new JPanel();
	private JTextArea dispArea = new JTextArea();
	private JButton buySell = new JButton("Buy/Sell");
	private JButton viewPrice = new JButton("Track Stocks");
	private JButton viewDetails = new JButton("View Stock History");
	private JButton viewLeaders = new JButton("View Leaders");
	private JButton setAlert = new JButton("Set Alert Threshold");
	private JButton logout = new JButton("Logout");
	private JButton deregister = new JButton("Deregister");
	private Portfolio port;
	private Order pending;
	private AccountScreenUpdater asu;
	
	public AccountScreen(Portfolio p)
	{
		super((JFrame)(null), false);
		port = p;
		pending = AccountManager.getOrders(p.getName());
		asu = new AccountScreenUpdater();
		setTitle(port.getName() + "'s Account");
		setBounds(400, 150, 500, 550);
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.X_AXIS));
		getContentPane().add(butPanel);
		getContentPane().add(dispPanel);
		dispPanel.add(dispArea);
		butPanel.setLayout(new GridLayout(7,1,0,5));
		butPanel.add(buySell);
		butPanel.add(viewPrice);
		butPanel.add(viewDetails);
		butPanel.add(viewLeaders);
		butPanel.add(setAlert);
		butPanel.add(logout);
		butPanel.add(deregister);
		buySell.addActionListener(this);
		viewPrice.addActionListener(this);
		viewDetails.addActionListener(this);
		viewLeaders.addActionListener(this);
		setAlert.addActionListener(this);
		logout.addActionListener(this);
		deregister.addActionListener(this);
		dispArea.setText(port.toString());
		dispArea.setFocusable(false);
		setVisible(true);
		asu.start();
	}
	
	public void actionPerformed(ActionEvent ev)
	{
		if(ev.getSource() == buySell)
		{
			new BuySellScreen();
		}
		if(ev.getSource() == viewPrice)
		{
			new StockWatcher();
		}
		if(ev.getSource() == viewDetails)
		{
			new StockDetailScreen();
		}
		if(ev.getSource() == viewLeaders)
		{
			new LeaderBoard(port.getName());
		}
		if(ev.getSource() == setAlert)
		{
			new SetAlertScreen();
		}
		if(ev.getSource() == logout)
		{
			AccountManager.logout(port);
			AccountManager.storeOrders(pending, port.getName());
			dispose();
			new LoginScreen();
		}
		if(ev.getSource() == deregister)
		{
			AccountManager.removeAccount(port.getName());
			dispose();
			new LoginScreen();
		}
	}
}