/*written by: Alex Sood
debugged by: Alex Sood*/
import java.sql.ResultSet;
import java.sql.SQLException;
public class Portfolio
{
	private static final double commission = 0.02;
	private String name;
	private ResultSet assets;
	Portfolio(String n, ResultSet a)
	{
		name = n;
		assets = a;
	}
	public String getName()
	{
		return name;
	}
	public double getValue()
	{
		double value = 0;
		try
		{
			assets.beforeFirst();
			while(assets.next())
			{
				String type = assets.getString("type");
				double amount = assets.getDouble("amount");
				if(type.equals("cash")) value += amount;
				else value += amount * StockList.getPrice(type);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		return value;
	}
	public String toString()
	{
		String msg = "Stock\t\t# Shares\n";
		double cash = 0;
		try
		{
			assets.beforeFirst();
			while(assets.next())
			{
				String type = assets.getString("type");
				double amount = assets.getDouble("amount");
				if(type.equals("cash")) cash = amount;
				else msg += type + "\t\t" + amount + "\n";
			}
			msg += "\nCash Balance: " + cash + "\n";
			msg += "Total Value: " + getValue();
		}
		catch(SQLException e)
		{
			e.printStackTrace();
			return null;
		}
		return msg;
	}
	public boolean sell(String stock, double amnt)
	{
		boolean found = false;
		int r = 0;
		double a = 0;
		try
		{
			double income = amnt * StockList.getPrice(stock);
			income -= income*commission;
			assets.beforeFirst();
			while(assets.next())
			{
				String type = assets.getString("type");
				double amount = assets.getDouble("amount");
				if(type.equals(stock) && amount > amnt)
				{
					assets.updateDouble("amount", amount-amnt);
					assets.updateRow();
					found = true;
				}
				else if(type.equals(stock) && amount == amnt)
				{
					assets.deleteRow();
					found = true;
				}
				if(type.equals("cash"))
				{
					assets.updateDouble("amount", amount + income);
					r = assets.getRow();
					a = amount;
					assets.updateRow();
				}
			}
			if(!found)
			{
				assets.absolute(r);
				assets.updateDouble("amount", a);
				assets.updateRow();
				new ErrBox("You don't own enough shares of that stock!");
			}
		}
		catch(SQLException e)
		{
			e.printStackTrace();
			return false;
		}
		return found;
	}
	public boolean buy(String stock, double amnt)
	{
		boolean found = false;
		int r = 0;
		double a = 0;
		try
		{
			double price = StockList.getPrice(stock);
			if(price == -1) return false;
			double payment = amnt*price;
			payment += payment*commission;
			assets.beforeFirst();
			while(assets.next())
			{
				String type = assets.getString("type");
				double amount = assets.getDouble("amount");
				if(type.equals(stock))
				{
					assets.updateDouble("amount", amount+amnt);
					r = assets.getRow();
					a = amount;
					assets.updateRow();
					found = true;
				}
				if(type.equals("cash") && amount >= payment)
				{
					assets.updateDouble("amount", amount - payment);
					assets.updateRow();
				}
				else if(type.equals("cash") && amount < payment)
				{
					assets.absolute(r);
					assets.updateDouble("amount", a);
					assets.updateRow();
					new ErrBox("You don't have enough cash!");
					return false;
				}
			}
			if(!found)
			{
				assets.moveToInsertRow();
				assets.updateDouble("amount",amnt);
				assets.updateString("name",name);
				assets.updateString("type",stock);
				assets.updateString("row_key", name + stock);
				assets.insertRow();
			}
		}
		catch(SQLException e)
		{
			e.printStackTrace();
			return false;
		}
		return true;
	}
}