/*written by: Alex Sood
debugged by: Alex Sood
debugging assisted by: John Grun*/
import com.sun.mail.smtp.SMTPSSLTransport;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.PrintStream;
import java.sql.SQLException;
import java.util.Properties;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.URLName;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.InternetAddress;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
public class SetAlertScreen extends JDialog implements ActionListener
{
	private static final int maxTrials = 5;
	private final String[] hosts = {"message.alltel.com", "txt.att.net", "messaging.nextel.com", "messaging.sprintpcs.com", "tmomail.net", "vtext.com"};

	private JComboBox carrier;
	private JTextField phoneField = new JTextField(15);
	private JTextField tickField = new JTextField(15);
	private JTextField priceField = new JTextField(15);
	private JTextField codeField = new JTextField(15);
	private JRadioButton upper = new JRadioButton("Upper Limit");
	private JRadioButton lower = new JRadioButton("Lower Limit");
	private JButton submit = new JButton("Submit");
	private JButton verify = new JButton("Verify");
	private JButton exit = new JButton("Exit");
	private JLabel phoneLabel = new JLabel("Enter 10 digit phone number (no dashes)");
	private JLabel tickLabel = new JLabel("Enter stock ticker");
	private JLabel priceLabel = new JLabel("Enter threshold price");
	private JLabel codeLabel = new JLabel("Enter verification code");
	private JLabel subLabel = new JLabel("Click Submit to receive verification code.");
	private int numTrials;
	private String code;
	
	public SetAlertScreen()
	{
		super((JFrame)(null), false);
		setTitle("SetAlertScreen");
		setBounds(200, 200, 350, 400);
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		getContentPane().add(phoneField);
		getContentPane().add(phoneLabel);
		String[] car = {"Select Carrier", "Alltel", "AT&T", "Nextel", "Sprint PCS", "T-Mobile", "Verizon"};
		carrier = new JComboBox(car);
		getContentPane().add(carrier);
		getContentPane().add(tickField);
		getContentPane().add(tickLabel);
		getContentPane().add(priceField);
		getContentPane().add(priceLabel);
		getContentPane().add(upper);
		getContentPane().add(lower);
		getContentPane().add(submit);
		getContentPane().add(subLabel);
		getContentPane().add(codeField);
		getContentPane().add(codeLabel);
		getContentPane().add(verify);
		getContentPane().add(exit);
		phoneField.setMaximumSize(new Dimension(500, 5));
		tickField.setMaximumSize(new Dimension(500, 5));
		priceField.setMaximumSize(new Dimension(500, 5));
		codeField.setMaximumSize(new Dimension(500, 5));
		phoneField.setAlignmentY(CENTER_ALIGNMENT);
		tickField.setAlignmentY(CENTER_ALIGNMENT);
		priceField.setAlignmentY(CENTER_ALIGNMENT);
		codeField.setAlignmentY(CENTER_ALIGNMENT);
		ButtonGroup bg = new ButtonGroup();
		bg.add(upper);
		bg.add(lower);
		lower.setSelected(true);
		submit.addActionListener(this);
		verify.addActionListener(this);
		exit.addActionListener(this);
		setVisible(true);
		numTrials = 5;
		try
		{
			BufferedReader fin = new BufferedReader(new FileReader("trl.txt"));
			numTrials = Integer.parseInt(fin.readLine());
		}
		catch(Exception e) { e.printStackTrace(); }
	}
	
	public void actionPerformed(ActionEvent ev)
	{
		if(ev.getSource() == exit)
		{
			try
			{
				PrintStream fout = new PrintStream(new FileOutputStream("trl.txt"));
				fout.print("" + numTrials + "\n");
			}
			catch(Exception e){ e.printStackTrace(); }
			dispose();
			return;
		}
		if(numTrials >= maxTrials)
		{
			new ErrBox("Set Alerts feature disabled. Please call 1-800-000-0000 for more information.");
			return;
		}
		String phone = phoneField.getText();
		if(phone.length() != 10)
		{
			new ErrBox("Please enter 10 digit phone number");
			return;
		}
		if(carrier.getSelectedIndex() == 0)
		{
			new ErrBox("Please select carrier");
			return;
		}
		String address = phone +"@" + hosts[carrier.getSelectedIndex() - 1];
		String ticker = tickField.getText();
		double price = -1;
		try
		{
			price = Double.parseDouble(priceField.getText());
		}
		catch(NumberFormatException e)
		{
			new ErrBox("Please enter threshold price.");
			return;
		}
		if(price <= 0)
		{
			new ErrBox("Threshold price must be positive.");
			return;
		}
		double cur_price = -1;
		try
		{
			cur_price = StockList.getPrice(ticker);
		}
		catch(SQLException e) { return; }
		int upLow = 0;
		if(lower.isSelected()) upLow++;
		if(upper.isSelected() && cur_price > price)
		{
			new ErrBox("Threshold already exceeded!");
			return;
		}
		if(lower.isSelected() && cur_price < price)
		{
			new ErrBox("Threshold already exceeded!");
			return;
		}
		if(ev.getSource() == submit)
		{
			numTrials++;
			code = "" + (int)(Math.random() * 90000 + 10000);
			sendCode(address);
		}
		if(ev.getSource() == verify)
		{
			if(code == null)
			{
				new ErrBox("Click Submit to receive code");
				return;
			}
			if(code.equals(codeField.getText()))
			{
				code = null;
				AccountManager.setThreshold(address, ticker, price, upLow);
				numTrials = 0;
			}
			else
			{
				new ErrBox("Code Invalid.");
			}
		}
	}
	
	private void sendCode(String addr)
	{
		try
		{
			Properties props = System.getProperties();
			props.put("mail.smtp.user", "asood");
			props.put("mail.smtp.host", "smtp.rutgers.edu");
			props.put("mail.smtp.starttls.enable","true");
			props.put("mail.smtp.auth", "true");
			Authenticator auth = new MyAuthenticator();
			Session ses = Session.getDefaultInstance(props, auth);
			ses.setDebug(false);
			SMTPSSLTransport trans = new SMTPSSLTransport(ses, new URLName("smtp", "smtp.rutgers.edu", 25, "", "asood", "hcrhs-06"));
			trans.setStartTLS(true);
			MimeMessage msg = new MimeMessage(ses);
			msg.setFrom(new InternetAddress("asood@rutgers.edu"));
			InternetAddress[] dest = {new InternetAddress(addr)};
			msg.setRecipients(Message.RecipientType.TO, dest);
			msg.setContent(code, "text/plain");
			Transport.send(msg);
		}
		catch(Exception e) { e.printStackTrace(); }
	}
	public class MyAuthenticator extends Authenticator
	{
		public PasswordAuthentication getPasswordAuthentication()
		{
			return new PasswordAuthentication("asood", "hcrhs-06");
		}
	}
}