/*written by: Alex Sood
debugged by: Alex Sood*/
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.InterruptedException;
import java.sql.SQLException;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
public class StockWatcher extends JDialog implements ActionListener
{
	
	public class StockWatchUpdater extends Thread
	{
		public StockWatchUpdater()
		{
			super("StockWatchUpdater");
			setDaemon(true);
		}
		public void run()
		{
			while(true)
			{
				for(int x = 0; x < watchList.length; x++)
				{
					if(watchList[x] != null)
					{
						try
						{
							String sq = StockList.getQuote(watchList[x]);
							if(sq != null) quotes[x].setText(sq);
						}
						catch(SQLException e)
						{
							e.printStackTrace();
						}
					}
				}
				try
				{
					sleep(5000);
				}
				catch(InterruptedException e)
				{
					System.out.println(e.toString());
				}				
			}
		}
	}
	
	private JPanel dispPanel = new JPanel();
	private JPanel inPanel = new JPanel();
	private JPanel fieldPanel = new JPanel();
	private JScrollPane dispPane = new JScrollPane(fieldPanel);
	private String[] watchList;
	private JTextField[] quotes;
	private JTextField header = new JTextField("Stock\tPrice\tChange");
	private JTextField input = new JTextField(15);
	private JButton go = new JButton("Go!");
	private JButton exit = new JButton("Exit");
	private int nextOpenField;
	private StockWatchUpdater updater;
	
	public StockWatcher()
	{
		super((JFrame)(null), false);
		setTitle("StockWatcher");
		setBounds(800, 300, 420, 230);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		getContentPane().add(dispPanel);
		getContentPane().add(inPanel);
		dispPanel.add(dispPane);
		dispPane.setPreferredSize(new Dimension(280, 135));
		dispPane.getVerticalScrollBar().setUnitIncrement(13);
		fieldPanel.setLayout(new GridLayout(21,1,0,0));
		header.setBackground(fieldPanel.getBackground());
		header.setFocusable(false);
		fieldPanel.add(header);
		watchList = new String[20];
		quotes = new JTextField[20];
		nextOpenField = 0;
		for(int x = 0; x < quotes.length; x++)
		{
			quotes[x] = new JTextField(20);
			if(x % 2 >0) quotes[x].setBackground(new Color(100, 100, 255));
			quotes[x].setFocusable(false);
			fieldPanel.add(quotes[x]);
		}
		inPanel.add(input);
		inPanel.add(go);
		inPanel.add(exit);
		go.addActionListener(this);
		input.addActionListener(this);
		exit.addActionListener(this);
		setVisible(true);
		updater = new StockWatchUpdater();
		updater.start();
	}
	
	public void actionPerformed(ActionEvent ev)
	{
		if(ev.getSource() == exit)
		{
			dispose();
		}
		else
		{
			String ticker = input.getText();
			input.setText("");
			for(int x = 0; x < nextOpenField; x++)
			{
				if(watchList[x].equals(ticker))
				{
					remove(x);
					return;
				}
			}
			add(ticker);
		}
	}
	public void remove(int pos)
	{
		nextOpenField--;
		for(int x = pos; x < nextOpenField; x++)
		{
			watchList[x] = watchList[x+1];
			quotes[x].setText(quotes[x+1].getText());
		}
		watchList[nextOpenField] = null;
		quotes[nextOpenField].setText("");
		input.requestFocusInWindow();
	}
	public void add(String tick)
	{
		if(nextOpenField == watchList.length)
		{
			System.out.println("Watch List is already full!");
			return;
		}
		try
		{
			String sq = StockList.getQuote(tick);
			if(sq != null)
			{
				quotes[nextOpenField].setText(sq);
				watchList[nextOpenField] = tick;
				nextOpenField++;
			}
		}
		catch(SQLException e)
		{
			e.printStackTrace();
		}
		input.requestFocusInWindow();
	}
}